@@ -1,5 +1,5 @@
 package ExtUtils::HasCompiler;
-$ExtUtils::HasCompiler::VERSION = '0.006';
+$ExtUtils::HasCompiler::VERSION = '0.007';
 use strict;
 use warnings;
 
@@ -8,7 +8,7 @@ our @EXPORT_OK = qw/can_compile_loadable_object/;
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
 use Config;
-use Carp 'croak';
+use Carp 'carp';
 use File::Basename 'basename';
 use File::Spec::Functions qw/catfile catdir/;
 use File::Temp qw/tempdir tempfile/;
@@ -71,37 +71,44 @@ sub can_compile_loadable_object {
 
 	my $shortname = '_Loadable' . $counter++;
 	my $package = "ExtUtils::HasCompiler::$shortname";
-	printf $source_handle $loadable_object_format, $basename, $package or croak "Couldn't write to file: $!";
-	close $source_handle or croak "Couldn't close sourcefile: $!";
+	printf $source_handle $loadable_object_format, $basename, $package or do { carp "Couldn't write to $source_name: $!"; return };
+	close $source_handle or do { carp "Couldn't close $source_name: $!"; return };
 
-	my ($cc, $ccflags, $optimize, $cccdlflags, $lddlflags, $libperl, $perllibs, $archlibexp) = map { $config->get($_) } qw/cc ccflags optimize cccdlflags lddlflags libperl perllibs archlibexp/;
-	my $incdir = catdir($archlibexp, 'CORE');
+	my $abs_basename = catfile($tempdir, $basename);
+	my $object_file = $abs_basename . $config->get('_o');
+	my $loadable_object = $abs_basename . $config->get('dlext');
+	my $incdir = catdir($config->get('archlibexp'), 'CORE');
 
-	my $loadable_object = catfile($tempdir, $basename . '.' . $config->get('dlext'));
+	my ($cc, $ccflags, $optimize, $cccdlflags, $ld, $lddlflags, $libperl, $perllibs) = map { $config->get($_) } qw/cc ccflags optimize cccdlflags ld lddlflags libperl perllibs/;
 
-	my $command;
+	my @commands;
 	if ($^O eq 'MSWin32' && $cc =~ /^cl/) {
 		require ExtUtils::Mksymlists;
-		my $abs_basename = catfile($tempdir, $basename);
 		#Mksymlists will add the ext on its own
 		ExtUtils::Mksymlists::Mksymlists(NAME => $basename, FILE => $abs_basename);
-		$command = qq{$cc $ccflags $optimize /I "$incdir" $source_name $abs_basename.def /Fo$abs_basename.obj /Fd$abs_basename.pdb /link $lddlflags $libperl $perllibs /out:$loadable_object};
+		push @commands, qq{$cc $ccflags $cccdlflags $optimize /I "$incdir" $source_name /Fo$object_file};
+		push @commands, qq{$ld $object_file $lddlflags $libperl $perllibs /out:$loadable_object /def:$abs_basename.def /pdb:$abs_basename.pdb};
 	}
 	elsif ($^O eq 'VMS') {
-		warn "VMS is currently unsupported";
+		carp "VMS is currently unsupported";
 		return;
 	}
 	else {
-		my $extra = $^O eq 'MSWin32' || $^O eq 'cygwin' ? '-l' . ($libperl =~ /lib([^.]+)\./)[0] : '';
+		my $extra = $^O eq 'MSWin32' ? '-l' . ($libperl =~ /lib([^.]+)\./)[0]
+			: $^O eq 'cygwin' ? catfile($incdir, $config->get('useshrplib') ? 'libperl.dll.a' : 'libperl.a')
+			: '';
 		if ($^O eq 'aix') {
 			$lddlflags =~ s/\Q$(BASEEXT)\E/$basename/;
 			$lddlflags =~ s/\Q$(PERL_INC)\E/$incdir/;
 		}
-		$command = qq{$cc $ccflags "-I$incdir" $cccdlflags $source_name $lddlflags $extra $perllibs -o $loadable_object };
+		push @commands, qq{$cc $ccflags $optimize "-I$incdir" $cccdlflags -c $source_name -o $object_file};
+		push @commands, qq{$cc $optimize $object_file -o $loadable_object $lddlflags $extra $perllibs};
 	}
 
-	print "$command\n" if not $args{quiet};
-	system $command and die "Couldn't execute $command: $!";
+	for my $command (@commands) {
+		print "$command\n" if not $args{quiet};
+		system $command and do { carp "Couldn't execute $command: $!"; return };
+	}
 
 	# Skip loading when cross-compiling
 	return 1 if exists $args{skip_load} ? $args{skip_load} : $config->get('usecrosscompile');
@@ -140,7 +147,7 @@ ExtUtils::HasCompiler - Check for the presence of a compiler
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 DESCRIPTION
 
